#import "IonicFile.h"
#import "CDVLocalFilesystem.h"
#import "CDVAssetLibraryFilesystem.h"
#import <objc/message.h>

@implementation IonicFile
@synthesize libPath, libPathSync, libPathNoSync, storagePath, docPath, cachePath, tmpPath, appBundlePath;

- (void)pluginInitialize
{
    libPath = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES)[0];
    storagePath = [libPath stringByDeletingLastPathComponent];
    docPath = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES)[0];
    cachePath = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES)[0];
    tmpPath = [NSTemporaryDirectory() stringByResolvingSymlinksInPath]; // in /private/var for some reason
    
    // Paths relative to storagePath
    NSString* libPathRelative = [libPath substringFromIndex:[libPath rangeOfString:storagePath].length];
    NSString* docPathRelative = [docPath substringFromIndex:[docPath rangeOfString:storagePath].length];
    NSString* cachePathRelative = [cachePath substringFromIndex:[cachePath rangeOfString:storagePath].length];
    NSString* tmpPathRelative = [tmpPath substringFromIndex:[tmpPath rangeOfString:storagePath].length];
    
    // get app variables
    SEL targetIDSelector = @selector(targetID);
    id targetID = objc_msgSend(self.viewController, targetIDSelector);
    id snapshotID = objc_msgSend(self.viewController, @selector(snapshotID));
    
    // Where the app files are stored
    appBundlePath = [NSString stringWithFormat:@"%@/NoCloud/files/%@/%@/", libPath, targetID, snapshotID];
    
    // tack targetID on to storagePath, then add relative paths back on
    storagePath = [NSString stringWithFormat:@"%@/NoCloud/files/%@", libPath, targetID];
    libPath = [storagePath stringByAppendingString:libPathRelative];
    docPath = [storagePath stringByAppendingString:docPathRelative];
    cachePath = [storagePath stringByAppendingString:cachePathRelative];
    tmpPath = [storagePath stringByAppendingString:tmpPathRelative];
    libPathSync = [libPath stringByAppendingPathComponent:@"Cloud"];
    libPathNoSync = [libPath stringByAppendingPathComponent:@"NoCloud"];
    NSString* docPathNoSync = [docPath stringByAppendingPathComponent:@"NoCloud"];
    
    // Create directories
    [[NSFileManager defaultManager] createDirectoryAtPath:libPathNoSync withIntermediateDirectories:YES attributes:nil error:nil];
    [[NSFileManager defaultManager] createDirectoryAtPath:docPathNoSync withIntermediateDirectories:YES attributes:nil error:nil];
    [[NSFileManager defaultManager] createDirectoryAtPath:tmpPath withIntermediateDirectories:YES attributes:nil error:nil];
    
    // No iCloud backup
    [[NSURL fileURLWithPath:libPathNoSync] setResourceValue: [NSNumber numberWithBool: YES]
                                                     forKey: NSURLIsExcludedFromBackupKey error:nil];
    
    [self registerFilesystem:[[CDVLocalFilesystem alloc] initWithName:@"temporary" root:tmpPath]];
    [self registerFilesystem:[[CDVLocalFilesystem alloc] initWithName:@"persistent" root:docPath]];
    [self registerFilesystem:[[CDVAssetLibraryFilesystem alloc] initWithName:@"assets-library"]];
    
    NSDictionary* filesystems =  @{
                                   @"library": libPath,
                                   @"library-nosync": libPathNoSync,
                                   @"documents": docPath,
                                   @"documents-nosync": docPathNoSync,
                                   @"cache": cachePath,
                                   @"bundle": appBundlePath
                                   };
    for (NSString* name in filesystems) {
        [self registerFilesystem:[[CDVLocalFilesystem alloc] initWithName:name root:filesystems[name]]];
    }
}

- (void)requestAllPaths:(CDVInvokedUrlCommand*)command
{
    NSDictionary* ret = @{
                          @"applicationDirectory": [[NSURL fileURLWithPath:appBundlePath] absoluteString],
                          @"applicationStorageDirectory": [[NSURL fileURLWithPath:storagePath] absoluteString],
                          @"dataDirectory": [[NSURL fileURLWithPath:libPathNoSync] absoluteString],
                          @"syncedDataDirectory": [[NSURL fileURLWithPath:libPathSync] absoluteString],
                          @"documentsDirectory": [[NSURL fileURLWithPath:docPath] absoluteString],
                          @"cacheDirectory": [[NSURL fileURLWithPath:cachePath] absoluteString],
                          @"tempDirectory": [[NSURL fileURLWithPath:tmpPath] absoluteString]
                          };
    
    CDVPluginResult* result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:ret];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

@end
